"""
A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open  # pylint: disable=redefined-builtin
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "pypi_README.md"), encoding="utf-8") as f:
    long_description = f.read()

exec(open(path.join(here, "mppsolar/version.py")).read())

setup(
    name="mpp-solar",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=__version__,  # noqa: F821
    description="Package to communicate with MPP Solar PIP-4048MS inverter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # The project's main homepage.
    url="https://github.com/jblance/mpp-solar",
    # Author details
    author="John Blance",
    author_email="",
    # Choose your license
    license="MIT",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires=">=3.6",
    # What does your project relate to?
    keywords="hardware serial communication",
    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=["contrib", "daemon", "docs", "tests"]),
    include_package_data=True,
    data_files=[
        ("/etc/systemd/user", ["daemon/mpp-solar.service"]),
        ("/etc/systemd/user", ["daemon/jkbms.service"]),
        ("/etc/mpp-solar", ["daemon/mpp-solar.conf.example"]),
        ("/etc/jkbms", ["daemon/jkbms.conf.example"]),
    ],
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    # install_requires=["pyserial", "paho-mqtt", "bluepy"],
    install_requires=["pyserial", "paho-mqtt"],
    # install_requires=[],
    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        "dev": ["check-manifest"],
        "test": ["coverage"],
    },
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        "console_scripts": [
            "mpp-solar=mppsolar:main",
            "jkbms=mppsolar:main",
        ],
    },
    test_suite="tests.get_tests",
)
